/* GeneTabl.h */

#ifndef __GeneTabl_h
#define __GeneTabl_h

#include <iostream>

using namespace std;

#include "ClString.h"
#include "SrtArray.h"

_CLASSDEF(GenesTable)

class GenesTable : public Clonable
  {

    _CLASSDEF(Element)

    class Element : public Sortable
      {

      public :

        String   species;
        SrtArray genes;

      public :

        Element( RCString  spName="" ) : Sortable(),
          species(spName),    genes(256)     {}
        Element( RCElement e         ) : Sortable(),
          species(e.species), genes(e.genes) {}
        ~Element() {}
        void flush() { species="";  genes.flush(); }

        OK_ERROR readFrom( istream& is );

        /*    Object'a */
        virtual classType   isA     () const { return UNDEFINED; }
        virtual const char* nameOf  () const { return "GenesTable::Element"; }
        virtual bool        isEqual ( RCObject o ) const
          { return ( o.nameOf()==nameOf() && species==RCElement(o).species ); }
        virtual void        printOn ( ostream& os ) const;

        /*  "" Clonable */
        virtual PClonable clone() const { return new Element(ME); }

        /*  "" Sortable */
        virtual bool isLessThan( RCSortable s ) const
          { return ( /*nameOf()==s.nameOf() &&*/
                     species<RCElement(s).species ); }

      };

  public :

    GenesTable() {}
    GenesTable( RCGenesTable gT ) : Clonable(), elements(gT.elements) {}
    ~GenesTable() {}

    OK_ERROR readFrom( istream& is );
    String   whatSpecies( RCString gene ) const;
    int      speciesNumber( RCString spec ) const;
    int      numberOfSpecies() const { return elements.size(); }
    String   species( int i )  const { return ME[i].species; }

    RCElement operator [] ( int i ) const
      { return RCElement(elements[i]); }

    /*    Object'a */
    virtual classType   isA     () const { return UNDEFINED; }
    virtual const char* nameOf  () const { return "GenesTable"; }
    virtual bool        isEqual ( RCObject o ) const
      { return (o.nameOf()==nameOf() && elements==RCGenesTable(o).elements); }
    virtual void        printOn ( ostream& os ) const
      { os << elements; }

    /*  "" Clonable */
    virtual PClonable clone() const { return new GenesTable(ME); }

  private :

    SrtArray elements;

  };

#endif // __GeneTabl_h

