/* IntMat.cpp */

#include "IntMat.h"

#define max(a,b) ( ( (a) > (b) ) ? (a) : (b) )
#define min(a,b) ( ( (a) < (b) ) ? (a) : (b) )
#define abs( a ) ( ( (a) >= 0  ) ? (a) : (-a) )

#define elem MatrixElement
#define imat IntMatrix
#define indx MatrixIndex

#define ERRORMESSAGE( MESSAGE ) { cerr << MESSAGE << endl; exit(1); }

//inline void swap( indx& a, indx& b ) { indx t=a; a=b, b=t; }
inline void swap( elem& a, elem& b ) { elem t=a; a=b, b=t; }

bool imat::operator==( const imat& A ) const
  {
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        if ( ME(i,j) != A(i,j) )
          return false;
    return true;
  }

bool imat::operator==( elem a ) const
  {
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        if ( ME(i,j) != (i==j)*a )
          return false;
    return true;
  }

imat& imat::operator=( const imat& A )
  {
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        ME(i,j) = A(i,j);
    return ME;
  }

imat imat::operator+( const imat& A ) const
  {
    imat T=ME;
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        T(i,j) += A(i,j);
    return T;
  }

imat imat::operator-( const imat& A ) const
  {
    imat T=ME;
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        T(i,j) -= A(i,j);
    return T;
  }

imat imat::operator+( elem a ) const
  {
    imat T=ME;
    for ( indx i=0; uns(i)<M && uns(i)<N; i++ )
      T(i,i) += a;
    return T;
  }

imat imat::operator-( elem a ) const
  {
    imat T=ME;
    for ( indx i=0; uns(i)<M && uns(i)<N; i++ )
      T(i,i) -= a;
    return T;
  }

imat imat::operator*( elem a ) const
  {
    imat T=ME;
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        T(i,j) *= a;
    return T;
  }

imat imat::operator/( elem a ) const
  {
    if ( !a ) ERRORMESSAGE("Attempt to devide matrix by ZERO! Exiting.")
    imat T=ME;
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        T(i,j) /= a;
    return T;
  }

imat& imat::operator+=( const imat& A )
  {
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        ME(i,j) += A(i,j);
    return ME;
  }

imat& imat::operator-=( const imat& A )
  {
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        ME(i,j) -= A(i,j);
    return ME;
  }

imat& imat::operator*=( elem a )
  {
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        ME(i,j) *= a;
    return ME;
  }

imat& imat::operator/=( elem a )
  {
    if ( !a ) ERRORMESSAGE("Attempt to devide matrix by ZERO! Exiting.")
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        ME(i,j) /= a;
    return ME;
  }

void imat::initialize()
  { for ( indx i=0; uns(i)<M*N; i++ ) el[i] = 0; }

imat imat::transp() const
  {
    imat T(N,M);
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        T(j,i) = ME(i,j);
    return T;
  }

void imat::swapLines( indx m1, indx m2 )
  {
    if ( m1<0 || m2<0 || uns(m1)>=M || uns(m2)>=M || m1==m2 )
      return;
    for ( indx k=0; uns(k)<N; k++ )
      swap( ME(m1,k), ME(m2,k) );
  }

void imat::swapColumns( indx n1, indx n2 )
  {
    if ( n1<0 || n2<0 || uns(n1)>=N || uns(n2)>=M || n1==n2 )
      return;
    for ( indx k=0; uns(k)<M; k++ )
      swap( ME(k,n1), ME(k,n2) );
  }

imat imat::subMatrix( indx i0, indx j0, indx i1, indx j1 ) const
  {
    if ( i0 > i1 ) swap( i0, i1 );
    if ( j0 > j1 ) swap( j0, j1 );
    if ( !indexesInRange(i0,j0) || indexesInRange(i1,j1) || i0==i1 || j0==j1 )
      return ( *(imat*)NULL );

    imat T(i1-i0+1,j1-j0+1);
    for ( indx i=0; uns(i)<T.M; i++ )
      for ( indx j=0; uns(j)<T.N; j++ )
        T(i,j) = ME(i0+i,j0+j);

    return T;
  }

bool imat::isDiagonal() const
  {
    if ( M!=N )
      return false;
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=0; uns(j)<N; j++ )
        if ( j!=i && ME(i,j)!=0 )
          return false;
    return true;
  }

bool imat::isSymmetric() const
  {
    if ( M!=N )
      return false;
    for ( indx i=0; uns(i)<M; i++ )
      for ( indx j=i+0; uns(j)<N; j++ )
        if ( ME(i,j) != ME(j,i) )
          return false;
    return true;
  }

void imat::printOn( ostream& os ) const
  {
    for ( indx i=0; uns(i)<M; i++, os << endl )
      for ( indx j=0; uns(j)<N; j++ )
        os << ME(i,j) << ( (uns(j)<N-1) ? "\t" : "" );
  }

