/* LogOut.cpp */

#include "LogOut.h"

#include <time.h>

#include "TInject.h"
#include "TreeMan.h"
#include "TreeOut.h"
#include "GlSttngs.h"

ofstream tlog;

inline void logOutTitle( const char* programName )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

//    struct date d; getdate(&d);
//    struct time t; gettime(&t);
    IMPORT_STRING( logFileName, "LogFileName" );
    String stars80=String('*')*80;
    String title=logFileName + ": Log-  " + programName + ".";
//    String curTime= String()
//      + int(d.da_day)  + '.' + int(d.da_mon)  + '.' + int(d.da_year) + ' '
//      + int(t.ti_hour) + ':' + int(t.ti_min)  + ':' + int(t.ti_sec)  + ','
//      + int(t.ti_hund);
    String titleLine=String("* ") + title
      + String(' ')*(76-title.length()/*-curTime.length()*/)
      + /*curTime +*/ " *";

    tlog << endl << stars80 << endl << titleLine << endl << stars80 << endl;
  }

/*   log-  */
void createLogFileHeader( int argc, char*const argv[],
                          const char* programName, const char* versionAndDate )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    logOutTitle( programName );
    tlog << endl << versionAndDate << endl << endl;

    tlog << " :" << endl;
    for ( int i=0; i<argc; i++ )
      tlog << argv[i] << ( (i<argc-1) ? " " : "\n" );
    tlog << endl;

    if ( GLOBAL_BOOLEAN("LogOutProgramSettings") )
      tlog << "---     :" << endl
           << globalSettings << endl;
    tlog << endl;

    return;
  }

/*  log- */
void finishLogFile( const char* programName )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    tlog << endl << " *    ." << endl;

    double finishTime=double(clock()+1)/CLOCKS_PER_SEC;
    tlog << endl << " *   < " << finishTime << " .\n";

    logOutTitle( programName );
  }


/*   log   */
void logOutInitialTrees( RCClArray gTrees, RCSpeciesTree sTree )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false ||
         GLOBAL_BOOLEAN("LogOutInitialTrees") == false )
      return;

    tlog << endl << "*****     .   *****" << endl;
    TreePrinter().printGeneTrees( gTrees, tlog );
    if ( !sTree.isTrivial() )
      tlog << endl << "*****      .   *****" << endl
           << sTree << ';' << endl;
    tlog << endl;
  }

/*   log     */
void logOutCutResults( RCClArray gTrees, int numberOfRemovedGenes )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    tlog << " *    ." << endl;
    if ( numberOfRemovedGenes > 0 )
      {
        tlog << " !      ." << endl;
        if ( GLOBAL_BOOLEAN("LogOutCutGeneTrees") )
          {
            tlog << endl << "*****    .   *****" << endl;
            TreePrinter().printGeneTrees( gTrees, tlog );
          }
      }
    else
      tlog << " *   --  ." << endl;
  }

/*   log    */
void logOutInitialInjections( RCClArray gTrees, RCSpeciesTree sTree,
                              double initialQuality )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    tlog << endl;
    if ( GLOBAL_BOOLEAN("LogOutInitialInjections") )
      {
        tlog << "   :" << endl;
        TreePrinter().printInjections( gTrees, sTree, tlog );
      }
    tlog << "    : "
         << initialQuality << endl << endl;
  }

/*   log   */
void logOutOptimizedInjections( RCClArray gTrees, RCSpeciesTree sTree,
                                double resultQuality )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    tlog << endl;
    if ( GLOBAL_BOOLEAN( "LogOutOptimizedInjections" ) )
      {
        tlog << "   :" << endl;
        TreePrinter().printInjections( gTrees, sTree, tlog );
      }
    tlog << "    : "
         << resultQuality << endl << endl;
  }

/*   log     */
void logOutNewTryHeader( int tryNum )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    tlog << endl << endl
         << "##########  " << (tryNum+1) << " ##########"
         << endl << endl;
  }

/*   log     */
void logOutNewRandomSpeciesTree( RCSpeciesTree sTree )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false ||
         GLOBAL_BOOLEAN("LogOutRandomSpeciesTrees") == false )
      return;

    tlog << "    :" << endl
         << sTree << ';' << endl << endl;
  }

/*   log    */
void logOutBestSpeciesTrees( RCSpeciesTree sTree,
                             RCSpeciesTreesArray bestSTrees,
                             int numOfBestSTrees )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false )
      return;

    tlog << endl << "   :" << endl << sTree << ';' << endl;
    if ( numOfBestSTrees != 1 )
      {
        if ( numOfBestSTrees > 1 )
          tlog << "   :" << endl;
        else
          tlog << "    :";
        tlog << endl << bestSTrees << endl;
      }
  }

//   log  ͣ ,    
void logOutAbsentGenes( RCSrtArray absentGenes, int numberOfAllGenes )
  {
    if ( GLOBAL_BOOLEAN("LogAllowed") == false ||
         GLOBAL_BOOLEAN("LogOutAbsentGenes") == false ||
         absentGenes.isEmpty()  )
      return;

    const int nOfAbsentGenes=absentGenes.size();
    tlog << " ͣ ,     ("
         << nOfAbsentGenes << "   " << numberOfAllGenes << "):" << endl;
    for ( int i=0; i<nOfAbsentGenes; i++ )
      tlog << absentGenes[i] << ( (i<nOfAbsentGenes-1) ? ", " : ".\n" );
  }

