/* TIQMMain.cpp -  main()  TIQMax */

#include "LogOut.h"    //    log-
#include "TUtils.h"    //     
#include "Analyse.h"   //  
#include "TreeOut.h"   //  
#include "CmdLnPrm.h"  //    
#include "DefGlSet.h"  //      . 
#include "NewHndlr.h"  //     
#include "OpenFile.h"  //     /
#include "Optimize.h"  //   ( )
#include "ReadTree.h"  //    
#include "ReadSets.h"  //      
#include "StrCnsts.h"  //   

int main( int argc, char* argv[] )
  {

    /*     *****************************************/

    /*  */
    cout << programName << ": " << programShortDescription << endl
         << versionAndDate << endl << endl;

    /*        */
    if ( argc>1 && ( argv[1][0]=='?' || argv[1][1]=='?' ||
                     String(argv[1]) == "-h" ||
                     String(argv[1]) == "--help" ) )
      {
        cout << usage << endl << helpMessage;
        return OK;
      }

    /*        */
    set_new_handler( my_new_handler );

    /*      */
    setDefaultGlobalSettings();  //    
    {
      Settings commandLineParameters=readCommandLineParameters( argc, argv );
        //    
      globalSettings.set( commandLineParameters );
        //    

      ifstream settingsFile;         //      
      ofstream defaultSettingsFile;  //      
      IMPORT_STRING( settingsFileName, "SettingsFileName");
        //   
      IMPORT_STRING( defaultSettingsFileName, "SettingsFileName" );
        //     
      if ( openFile( settingsFile, settingsFileName ) )
        readGlobalSettings(settingsFile);       //  
      else if ( openFile( settingsFile, defaultSettingsFileName ) )
        readGlobalSettings(settingsFile);       //     
      else if ( openFile( defaultSettingsFile, defaultSettingsFileName ) )
        defaultSettingsFile << globalSettings;  //     

      globalSettings.set( commandLineParameters );
        //      
    }

    /*  ( ) log- */
    if ( GLOBAL_BOOLEAN("LogAllowed") )
      {
        globalSettings.set( "LogAllowed", false );  //   log
        IMPORT_STRING( logFileName, "LogFileName" );
        if ( logFileName!="NUL" && logFileName!="Nul" && logFileName!="nul" )
          {
            if ( !openFile( tlog, logFileName ) )
              return ERROR;
            globalSettings.set( "LogAllowed", true );  //  log
            createLogFileHeader( argc, argv, programName, versionAndDate );
              //   log- 
          }
      }
    IMPORT_BOOLEAN( logAllowed, "LogAllowed" );

    /*   */
    if ( GLOBAL_BOOLEAN( "InjectOnly" ) &&
         GLOBAL_INTEGER( "NumberOfRandomSpeciesTrees" ) != 0 )
      {
        globalSettings.set( "NumberOfRandomSpeciesTrees", 0 );
        if ( logAllowed )
          tlog << endl << " NumberOfRandSpeciesTrees   0 "
               << "   ,   InjectOnly  true." << endl;
      }
    //    ((?)) ݣ   ...


    /*   ********************************************************/

    cout << " ... " << flush;

    /*      */
    GenesTable gTable;  //    
    CHECK( readGenesTable(gTable), "    ." );

    /*    */
    ClArray gTrees;  //     
    CHECK( readGeneTrees(gTrees), "    ." );
    if ( gTrees.isEmpty() )
      RETURN_ERROR( "       (?)" );

    /*    ( ) */
    SpeciesTree sTree;  //  
    if ( GLOBAL_INTEGER( "NumberOfRandomSpeciesTrees" ) == 0 )
      switch ( readSpeciesTree(sTree) )
        {
          case ERROR :
            RETURN_ERROR( "    ." );
          case 0 :
            RETURN_ERROR( "     (?)" );
        }

    /*     log-     */
    logOutInitialTrees( gTrees, sTree );

    cout << "Ok." << endl;


    /*    *******************************************/

    cout << " ... " << flush;

    /* ""     */
    setSpeciesOnGeneTrees( gTrees, gTable );

    /*         */
    if ( GLOBAL_INTEGER("NumberOfRandomSpeciesTrees") > 0 )
      generateSpeciesTree( gTrees, sTree );

    /*       */
    if ( GLOBAL_BOOLEAN( "CheckGeneTrees" ) == true )
      {
        if ( logAllowed )
          tlog << endl << " *   ." << endl;
        int numberOfRemovedGenes=removeAbsentGenes( gTrees, sTree );
        if ( numberOfRemovedGenes != 0 )
          cout << "! " << numberOfRemovedGenes << "   ! " << flush;
        if ( gTrees.size() < 1 )
          RETURN_ERROR( "      (?)" );
        logOutCutResults( gTrees, numberOfRemovedGenes );
      }
    else
      if ( logAllowed )
        tlog << endl << " *    ." << endl;

    /*  ,    */
    if ( GLOBAL_BOOLEAN("MergeOneSpeciesSubtrees") )
      {
        if ( logAllowed )
          tlog << " *     ." << endl;
        for ( int i=0; i<gTrees.size(); i++ )
          (RGeneTree(gTrees[i])).mergeOneSpeciesSubrees();
      }

    /*     ң   ( ޣ ) */
    if ( GLOBAL_BOOLEAN("IgnoreLengths") == true )
      equalizeLengths( gTrees );  //    ң  
    else
      normalizeGeneTrees( gTrees );  //   ң    

    /*     */
    IntMatrix proximity( gTable.numberOfSpecies(), gTable.numberOfSpecies() );
    if ( GLOBAL_INTEGER("NumberOfRandomSpeciesTrees") > 0 &&
         GLOBAL_BOOLEAN("UseEmpiricalProbability") )
      calculateProximity( proximity, gTrees, gTable );

    if ( logAllowed && GLOBAL_BOOLEAN( "LogOutPreparedTrees" ) )
      {
        tlog << endl << "   :" << endl;
        TreePrinter().printGeneTrees( gTrees, tlog );
        tlog << endl;
      }

    if ( logAllowed &&  
         GLOBAL_BOOLEAN( "UseEmpiricalProbability" ) &&
         GLOBAL_BOOLEAN( "LogOutProximityMatrix" ) )
      tlog << endl << "   "
                   <<"(      ):"
           << endl << proximity << endl;

    /*        */
    if ( GLOBAL_BOOLEAN( "ProduceGeneTreesFiles" ) )
      {
        if ( logAllowed )
          tlog << endl << " *      ."
               << endl;
        for ( int n=0; n<gTrees.size(); n++ )
          {
            RCGeneTree gTree=RCGeneTree(gTrees[n]);
            ofstream gTreeFile;
            if ( openFile( gTreeFile, gTree.name()+".ph" ) )
              gTreeFile << gTree << ';';
          }
      }

    cout << "Ok." << endl;


    /*   *****************************************************/

    /*   */
    if ( GLOBAL_INTEGER("NumberOfRandomSpeciesTrees") != 0 )
      CHECK( optimizeRandomSpeciesTrees( gTrees, sTree, gTable, proximity ),
             ":-(" )
    else
      CHECK( optimizeGivenSpeciesTree( gTrees, sTree ), ":-(" )

    /*     (  ) */
    if ( GLOBAL_BOOLEAN( "LookForBadGenes" ) ||
         GLOBAL_BOOLEAN( "AnalyseEvolutionaryEvents" ) )
      {
        cout << endl << " ,  ... " << flush;
        CHECK( analyseInjections( gTrees, sTree, gTable ),
               "  ." );
        cout << "Ok." << endl;
      }


    /*    ****************************************/

    /*  log- */
    if ( logAllowed )
      finishLogFile( programName );

    /*  */
    cout << endl << goodByeMessage;

    return OK;  // Happy END!

  }
