/* TModify.cpp -    TreeModifier */
/* !        . */
/*
()
() **************************************************************************
() * -inline   TreeModifier                                   *
() **************************************************************************
()
*/

#include "TModify.h"  //   TreeModifier

/*  */
TreeModifier::TreeModifier( RTree tree ) :
  TreeManipulator(tree),
  nOfVars(0),
  startVarN(0),
  curVarN(0)
  {
    switch ( initializeSubtrees() )
      {
      default :
      case  0 : nOfVars =  0;  break;
      case  3 : nOfVars =  3;  break;
      case  5 : nOfVars = 15;  break;
      }
    //        ---n-1      
    // N(n) = | | (2i-1)  
    //        - -i=1      n  ()
  }

/*
()       : makeModification()  ( );
()    :       ;
()   : ---;
()  : ---.
()
*/
void TreeModifier::makeModification()  //   >;-)
  {
    if ( isUnabled() )
      return;

    switch ( nOfVars )
      {
      case  3 :
        switch ( curVarN )
          {
          case 0 : swapSubtrees(0,1); break;
          case 1 : swapSubtrees(1,2); break;
          case 2 : swapSubtrees(2,0); break;
          }
        if ( curVarN == 2 )
          swapSubtrees(1,2);
        break;
      case 15 :
        switch ( curVarN )
          {
          case 13 : case 14 :
          case  0 : case  5 : case  9 : swapSubtrees(2,3); break;
          case  1 : case  6 : case 10 : swapSubtrees(3,4); break;
          case  3 : case  7 : case 11 : swapSubtrees(4,1); break;
          case  4 : case  8 : case 12 : swapSubtrees(1,2); break;
          }
        if ( curVarN == 2 || curVarN == 14 )
          swapSubtrees(0,1);
        break;
      }

    curVarN = correctIndex(curVarN+1);
  }

/*
()       : makeModification( variantNumber )  ( );
()    :       ;
()   :     ;
()  : ---.
()
*/
void TreeModifier::makeModification( int i )
  {
    unsigned modNum=correctIndex(i);  // 0<=modNum<nOfVars
    while ( modificationNumber() != modNum )
      makeModification();
  }

/*
[]       : initializeSubtrees()  ( );
[]    :     ;
[]   : ---;
[]  :  .
[]
*/
int TreeModifier::initializeSubtrees()
  {

    //     
    int caseNumber=-1, //   (. )
        i=0, j=0;      //      >= 2 
    // : "(j-) i-" -- (j-)  i-  ( )
    if ( host().numberOfChildren() >= 2 )               // >=2 
      if ( host().child(0).numberOfChildren() >= 2 )    // >=2 0-
        if ( host().child(1).numberOfChildren() >= 2 )  // >=2 1-
          caseNumber = 0;  //    
        else // child(1).numberOfChildren() <= 1        // < 2 1-
          if ( host().child(0).child(0).numberOfChildren() >= 2 )
            caseNumber = 1, i = 0, j = 0;
          else  // host().child(0).child(0).numberOfChildren() < 2
            if ( host().child(0).child(1).numberOfChildren() >= 2 )
              caseNumber = 1, i = 0, j = 1;
            else // host().child(0).child(1).numberOfChildren() < 2
              caseNumber = 2, i = 0;  // 2  0-
      else // host().child(0).numberOfChildren() < 2    // < 2 0-
        if ( host().child(1).numberOfChildren() >= 2 )  // >=2 1-
          if ( host().child(1).child(0).numberOfChildren() >= 2 )
            caseNumber = 1, i = 1, j = 0;
          else  // host().child(1).child(0).numberOfChildren() < 2
            if ( host().child(1).child(1).numberOfChildren() >= 2 )
              caseNumber = 1, i = 1, j = 1;
            else
              caseNumber = 2, i = 1;  // 2  1-

    uint numberOfSubtrees=0;  //  

    // 
    startVarN = curVarN = 0;
    for ( int i=0; i<5; i++ )
      subtrees[i] = NULL;

    switch ( caseNumber )
      {
      case 0 : //    
        numberOfSubtrees = 5;                     // 5 :
        subtrees[0] = &host().child(1);           // 1- ,
        subtrees[1] = &host().child(0).child(0);  // 0- 0-,
        subtrees[2] = &host().child(0).child(1);  // 1- 0-,
        subtrees[3] = &host().child(1).child(0);  // 0- 1-,
        subtrees[4] = &host().child(1).child(1);  // 1- 1-;
        break;
      case 1 : // (!i)-  - , (!j)- i- - , j- i- -  
        startVarN = 13;
        numberOfSubtrees = 5;                     // 5 
        subtrees[0] = &host().child(i).child(j);
        subtrees[1] = &host().child(!i);
        subtrees[2] = &host().child(i).child(!j);
        subtrees[3] = &host().child(i).child(j).child(0);
        subtrees[4] = &host().child(i).child(j).child(1);
        break;
      case 2 : // (!i)-  - ,  i- - 
        numberOfSubtrees = 3;                     // 3 :
        subtrees[0] = &host().child(!i);          // (!i)- ,
        subtrees[1] = &host().child(i).child(0);  // 0- i-,
        subtrees[2] = &host().child(i).child(1);  // 1- i-;
        break;
      default: break;
      }

    curVarN = startVarN;

    return numberOfSubtrees;  //    
  }

/*
[]       : swapSubtrees(i,j)  (  );
[]    :   i-      j-;
[]   :  (   ) ;
[]  : ---.
[]
*/
void TreeModifier::swapSubtrees( int i, int j )
  {
    TreeManipulator treeMan1( *subtrees[i] );  //   TreeMan'
    TreeManipulator treeMan2( *subtrees[j] );
    treeMan1.goToParent();                     //   
    treeMan2.goToParent();
    treeMan1.interchangeChildren(treeMan2);    //   
  }

